/*#################################################################################################
	For		:	Xmega 
	Version	:	1.0
	Date	:	30.07.2013
	Autor	:	Martin Junghans
	Hardware:	ATXmega, 32MHz at 16Mhz extern, Wiznet, Webserver
//################################################################################################*/

#include "tcp.h"

//#################################### SPI
SPI_Master_t			spiMasterE;
SPI_DataPacket_t		dataPacket;
WIZ_data_t wiznet;

void tcp_create_msg(void);
void tcp_parse_msg(void);

//##########################################################################################
ISR(SPIE_INT_vect)		{SPI_MasterInterruptHandler(&spiMasterE);	}
	
//##########################################################################################
void tcp_init(void)
{	
	//####################################### Externe Interrupts PortE 
// 	cbi(wiz_data->port_cs->DIR,WIZNET_INT);												// Eingang    
//  wiz_data->port_cs->PIN1CTRL	= PORT_OPC_WIREDORPULL_gc | PORT_ISC_FALLING_gc;		// PullDown, fallende Flanke
// 	sbi(wiz_data->port_cs->INT0MASK,WIZNET_INT);											// Interrupt freigeben
// 	wiz_data->port_cs->INTCTRL	= PORT_INT0LVL_LO_gc;										// Interrupt Level
	
	//####################################### Initialize SPI Master
	sbi(PORTE.DIR, 4);	sbi(PORTE.OUT, 4);		// output CS high (No slave addressed)		
	SPI_MasterInit(&spiMasterE, &SPIE, &PORTE, false, SPI_MODE_0_gc, PMIC_HILVLEN_bm, false, SPI_PRESCALER_DIV4_gc);

	//####################################### Initialize Wiznet Hardware
	wiznet.hw_port		= &PORTE;
	wiznet.hw_spi		= &spiMasterE;
	wiznet.hw_pin_cs	= 4;
	wiznet.hw_pin_reset	= 0;
			
	//####################################### Initialize Wiznet Mode
	// gateway for WIZnet
	memcpy(wiznet.tcp_gtw_addr, (uint8_t []){192,168,  0,  1				}, sizeof (wiznet.tcp_gtw_addr));	
	// subnet mask for WIZnet
	memcpy(wiznet.tcp_sub_mask, (uint8_t []){255,255,255,  0				}, sizeof (wiznet.tcp_sub_mask));	
	// IP address for WIZnet - base		
	memcpy(wiznet.tcp_ip_addr , (uint8_t []){192,168,  0, 10				}, sizeof (wiznet.tcp_ip_addr ));	
	// gateway for WIZnet	
	memcpy(wiznet.tcp_mac_addr, (uint8_t []){0x00,0x16,0x36,0xDE,0x58,0xF6	}, sizeof (wiznet.tcp_mac_addr));	
	// socket port number - base
	wiznet.tcp_port	= 80;	
	
	wiz_init(&wiznet);
}

	
//########################################################################################## tcp_run
void tcp_run(void)
{
	switch(wiz_read(&wiznet, S0_CR, S_STATUS_REG)) 
	{
	case SR_CLOSED:
		if (wiz_sock_init(&wiznet, S0_CR,MR_TCP, wiznet.tcp_port) > 0)	
			wiz_sock_listen(&wiznet, S0_CR);
		break;

	case SR_ESTABLISHED:
		while(IR_RECV == wiz_read(&wiznet, S0_CR, S_INTE_REG));

		//#################### READ the received message
		uint16_t rsize = wiz_sock_recvSize(&wiznet, S0_CR);						// Get the client request size		
		if (wiz_sock_recv(&wiznet, S0_CR,wiznet.bufRXTX,rsize) < 1)		
			break;
			
		// rs232_USB_puts("Content:\r\n"); rs232_USB_puts(wiznet.bufRXTX); rs232_USB_puts("\r\n");	// DEBUG
					
		//#################### PARSE message
		tcp_parse_msg();		
						
		//#################### SEND Answer
		tcp_create_msg();			


		// clear interrupt flag
		wiz_write(&wiznet, S0_CR, S_INTE_REG, IR_ACK);
		while(wiz_read(&wiznet, S0_CR, S_INTE_REG) & IR_ACK)
		{
			;
		}
			
		// Now Send the HTTP Remaining Response
		if (wiz_sock_send(&wiznet, S0_CR) <= 0)	
			break;	
				
		wiz_sock_disconnect(&wiznet, S0_CR);						// Disconnect the socket
		break;

	case SR_FIN_WAIT:
	case SR_CLOSING:
	case SR_TIME_WAIT:
	case SR_CLOSE_WAIT:
	case SR_LAST_ACK:		
		wiz_sock_close(&wiznet, S0_CR);								// Force to close the socket																	// DEBUG
		break;
	}	
}	

//########################################################################################## tcp_create_msg
void tcp_create_msg(void)
{				
	// Create the HTTP Response	Header/Body
	strcpy_P(wiznet.bufRXTX,PSTR("<html><body>"));
	strcat_P(wiznet.bufRXTX,PSTR("<h3>Embedded AVR Xmega128A3 Server </h3>"));
	strcat_P(wiznet.bufRXTX,PSTR("<p>"));

	// Create the HTTP Response Content
	strcat_P(wiznet.bufRXTX,PSTR("<br/>U:" )); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer); 				
	strcat_P(wiznet.bufRXTX,PSTR("<br/>I:" )); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer);
	strcat_P(wiznet.bufRXTX,PSTR("<br/>P:" )); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer);								
	strcat_P(wiznet.bufRXTX,PSTR("<br/>T1:")); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer);
	strcat_P(wiznet.bufRXTX,PSTR("<br/>T2:")); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer);
	strcat_P(wiznet.bufRXTX,PSTR("<br/>T3:")); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer);
	strcat_P(wiznet.bufRXTX,PSTR("<br/>T4:")); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer);
	strcat_P(wiznet.bufRXTX,PSTR("<br/>T5:")); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer);
	strcat_P(wiznet.bufRXTX,PSTR("<br/>T6:")); itoa(12345, buffer,10); strcat  (wiznet.bufRXTX,buffer);
				
	// Create the HTTP Response Footer
 	strcat_P(wiznet.bufRXTX,PSTR("</p></body></html>"));
}
//########################################################################################## tcp_parse
void tcp_parse_msg(void)
{
	// parse the new msg	
	// wiznet.bufRXTX[]
}	

